library(fda)
library(ggplot2)

data(CanadianWeather)
temp = CanadianWeather$dailyAv[,,1]
precip = CanadianWeather$dailyAv[,,2]

daytime = (1:365)-0.5
day5 = seq(0,365,5)


### Plot these data
df <- data.frame(daytime = daytime, temp = c(temp), precip = c(precip),
                 place = rep(CanadianWeather$place, each = 365), region = rep(CanadianWeather$region, each = 365))


# vykresleni teploty ve mestech - surova data
x11()
p0 <- ggplot (data = df, aes (x = daytime, y = temp, colour = place))
p0 <- p0 + geom_point (size = 0.5)
p0 <- p0 + labs (x = "Days", y = "Temperature") 
p0 <- p0 + theme_bw ()
print (p0)

dev.copy2pdf (file = "temperature_data.pdf", width = 12, height = 7)

# vykresleni teploty ve mestech - krivky
x11()
p0 <- ggplot (data = df, aes (x = daytime, y = temp, colour = place))
p0 <- p0 + geom_line ()
p0 <- p0 + labs (x = "Days", y = "Temperature") 
p0 <- p0 + theme_bw ()
print (p0)

dev.copy2pdf (file = "temperature.pdf", width = 12, height = 7)

# We can also plot by region; Atlantic, Pacific, Central and North. 
# vykresleni teploty v provinciich
x11()
p0 <- ggplot (data = df, aes (x = daytime, y = temp, colour = region, group = place))
p0 <- p0 + geom_line ()
p0 <- p0 + labs (x = "Days", y = "Temperature") 
p0 <- p0 + theme_bw ()
print (p0)

dev.copy2pdf (file = "temperature_region.pdf", width = 12, height = 7)

# vykresleni srazek ve mestech
x11()
p0 <- ggplot (data = df, aes (x = daytime, y = precip, colour = place))
#p0 <- p0 + geom_point ()
#p0 <- p0 + guides (colour = FALSE) 
p0 <- p0 + geom_line ()
p0 <- p0 + labs (x = "Days", y = "Precipitation") 
p0 <- p0 + theme_bw ()
#p0 <- p0 + scale_x_continuous (breaks = 1:365)
print (p0)

dev.copy2pdf (file = "precipitation.pdf", width = 12, height = 7)

################################################################################
###                          DEFINING A BASIS SYSTEM                        ####
################################################################################

dayrng = c(0,365)

#### 1. Fourier Basis with 365 basis functions
fbasis = create.fourier.basis(dayrng,365)

# Plot a basis with just 5 components

plot(create.fourier.basis(dayrng,5))

# etc

#### 2. B-spline bases with knots every 5 days

knots = day5
norder = 4
nbasis = length(knots) + norder - 2

bbasis = create.bspline.basis(dayrng,nbasis,norder,knots)

# Plotting these is a little crazy

plot(bbasis)

# but we can look at a smaller number

plot(create.bspline.basis(dayrng,nbasis=12,norder))


################################################################################
###                          SMOOTHING FUNCTIONS                            ####
################################################################################

#### 1. Lfd Objects

# Two common means of generating Lfd objects
# 1. int2Lfd -- just penalize some derivatives. 

curv.Lfd = int2Lfd(2)

# 2. vec2Lfd -- a (constant) linear combination of derivatives; for technical
# reasons this also requires the range of the basis. 

harmLfd = vec2Lfd(c(0,(2*pi/365)^2,0),rangeval=dayrng)

#### 2. fdPar objects

# We'll concentrate on B-splines and second-derivative penalties.

lambda = 1e6

# Now we can define the fdPar object 

curv.fdPar = fdPar(bbasis,curv.Lfd,lambda)

#### 4. Choosing smoothing parameters

# We can search through a collection of smoothing parameters to try and find
# an optimal parameter.

# We will record the average gcv and choose lambda to be the minimum of these. 

lambdas = 10^seq(-4,4,by=0.5)    # lambdas to look over

mean.gcv = rep(0,length(lambdas)) # store mean gcv


for(ilam in 1:length(lambdas)){
  # Set lambda
  curv.fdPari = curv.fdPar
  curv.fdPari$lambda = lambdas[ilam]
  
  # Smooth
  tempSmoothi = smooth.basis(daytime,temp,curv.fdPari)
  
  # Record average gcv
  mean.gcv[ilam] = mean(tempSmoothi$gcv)
}

# Lets select the lowest of these and smooth

best = which.min(mean.gcv)
lambdabest = lambdas[best]

curv.fdPar$lambda = lambdabest
tempSmooth = smooth.basis(daytime,temp,curv.fdPar)

### Plot smooth data
tempfd <- tempSmooth$fd
fdobjSmootheval <- eval.fd(fdobj = tempfd, evalarg = daytime)

dfs <- data.frame(daytime = daytime, temp = c(fdobjSmootheval), precip = c(precip),
                 place = rep(CanadianWeather$place, each = 365), region = rep(CanadianWeather$region, each = 365))

# vykresleni teploty ve mestech
x11()
p0 <- ggplot (data = dfs, aes (x = daytime, y = temp, colour = place))
p0 <- p0 + geom_line ()
p0 <- p0 + labs (x = "Days", y = "Temperature") 
p0 <- p0 + theme_bw ()
print (p0)

dev.copy2pdf (file = "temperature_smooth.pdf", width = 12, height = 7)

################################################################################
###                   FUNCTIONAL PRINCIPAL COMPONENTS                       ####
################################################################################

#### 1. pca.fd

tempPCA = pca.fd(tempfd,nharm=6)

# Here we can look at proportion of variance explained:

df <- data.frame(Components = 1:6, varprop = cumsum(tempPCA$varprop))

x11()
p0 <- ggplot (data = df, aes (x = Components, y = varprop))
p0 <- p0 + geom_line ()
p0 <- p0 + geom_point ()
p0 <- p0 + labs (x = "Components", y = "Variable Explained") 
p0 <- p0 + scale_x_continuous (breaks = 1:6)
p0 <- p0 + theme_bw ()
print (p0)

dev.copy2pdf (file = "varprop_canad.pdf", width = 12, height = 7)

## Looking at the principal components:

fdobjPCAeval <- eval.fd(fdobj = tempPCA$harmonics[1:3], evalarg = daytime)

df <- data.frame(daytime = daytime, harmonics = c(fdobjPCAeval), component = factor(rep(1:3, each = 365)))

x11()
p0 <- ggplot (data = df, aes (x = daytime, y = harmonics, color = component, linetype = component))
p0 <- p0 + geom_line ()
p0 <- p0 + labs (x = "Days", y = "Main Components") 
p0 <- p0 + theme_bw ()
print (p0)

dev.copy2pdf (file = "harmonics_canad.pdf", width = 12, height = 7)

# 1 Looks like over-all temperature. 
# 2 Looks like Summer vs Winter
# 3 Is Spring vs Fall. 

##### 2. PCA and Smoothing

pca.fdPar = fdPar(bbasis,curv.Lfd,1e4)

tempPCAsmooth = pca.fd(tempfd,nharm=6,harmfdPar=pca.fdPar)

# Let's plot the PCs

fdobjPCAsmootheval <- eval.fd(fdobj = tempPCAsmooth$harmonics[1:3], evalarg = daytime)

df <- data.frame(daytime = daytime, harmonics = c(fdobjPCAsmootheval), component = factor(rep(1:3, each = 365)))

x11()
p0 <- ggplot (data = df, aes (x = daytime, y = harmonics, color = component, linetype = component))
p0 <- p0 + geom_line ()
p0 <- p0 + labs (x = "Days", y = "Main Components") 
p0 <- p0 + theme_bw ()
print (p0)

dev.copy2pdf (file = "harmonics_smooth_canad.pdf", width = 12, height = 7)

##### 3. PCA and Reconstructing Data

mtempfd <- mean(tempfd)

scores <- tempPCAsmooth$scores
PCs <- eval.fd(fdobj = tempPCAsmooth$harmonics, evalarg = daytime)
fdm <- eval.fd(fdobj = mtempfd, evalarg = daytime)
colnames(fdm) <- NULL

# Firstly, just the mean
df <- data.frame(dfs, fdmean = fdm)

x11()
p0 <- ggplot (data = df, aes (x = daytime, y = temp, group = place))
p0 <- p0 + geom_line (color = "grey", size = 0.5)
p0 <- p0 + geom_line (aes(y = fdmean), color = 2, size = 1.5)
p0 <- p0 + labs (x = "Days", y = "Mean Curve") 
p0 <- p0 + theme_bw ()
print (p0)

dev.copy2pdf (file = "mean_canad.pdf", width = 4, height = 3)

# taky rozptyl

temp.varbifd = var.fd(tempfd)
temp.var = eval.bifd(daytime,daytime,temp.varbifd)
temp.cor = cor.fd(daytime,tempfd)

df <- data.frame(dfs, fdmean = fdm, fdvar = diag(temp.var))

x11()
p0 <- ggplot (data = df, aes (x = daytime, y = fdvar))
p0 <- p0 + geom_line (color = 2, size = 1.5)
p0 <- p0 + labs (x = "Days", y = "Variance Curve") 
p0 <- p0 + theme_bw ()
print (p0)

dev.copy2pdf (file = "var_canad.pdf", width = 4, height = 3)

x11()
p0 <- ggplot (data = df, aes (x = daytime, y = temp, group = place))
p0 <- p0 + geom_line (color = "grey", size = 0.5)
p0 <- p0 + geom_line (aes(y = fdmean), color = 2, size = 1.5)
p0 <- p0 + geom_line (aes(y = fdmean + sqrt(fdvar)), color = 3, size = 1)
p0 <- p0 + geom_line (aes(y = fdmean - sqrt(fdvar)), color = 3, size = 1)
p0 <- p0 + labs (x = "Days", y = "Mean Curve") 
p0 <- p0 + theme_bw ()
print (p0)

# kovariance a korelace

df <- merge(daytime, daytime)
df <- data.frame(df, fdcov = c(temp.var), fdcor = c(temp.cor))

x11()
p0 <- ggplot (data = df, aes (x, y, z = fdcov))
p0 <- p0 + geom_raster(aes(fill = fdcov))
p0 <- p0 + geom_contour (colour = "white")
p0 <- p0 + labs (x = "Days", y = "Days") 
p0 <- p0 + coord_fixed (ratio = 1)  
p0 <- p0 + theme_bw ()
print (p0)

dev.copy2pdf (file = "cov_canad.pdf", width = 12, height = 7)

# Mostly high variance in the winter, low in summer. Let's have a look at 
# correlation. In this case, evaluation arguments go in with the function call

x11()
p0 <- ggplot (data = df, aes (x, y, z = fdcor))
p0 <- p0 + geom_raster(aes(fill = fdcor))
p0 <- p0 + geom_contour (colour = "white")
p0 <- p0 + labs (x = "Days", y = "Days") 
p0 <- p0 + coord_fixed (ratio = 1)  
p0 <- p0 + theme_bw ()
print (p0)

dev.copy2pdf (file = "cor_canad.pdf", width = 12, height = 7)


filled.contour(day5,day5,temp.cor)

# Here we see high correlation between Summer and Winter temperatures, but 
# much less in the spring and fall (although spring to fall correlation is still
# high). 


# rekonstrukce
x11()
for (k in 1:6){
  df <- data.frame(dfs, reconstruction =  rep(fdm, times = 35) + c(PCs[, 1:k] %*% t(scores[, 1:k])))
  p0 <- ggplot (data = df, aes (x = daytime, y = temp, group = place))
  p0 <- p0 + geom_line (color = "grey", size = 0.5)
  p0 <- p0 + geom_line (aes(y = reconstruction, color = place))
  p0 <- p0 + labs (x = "Days", y = "Reconstructed Curves") 
  p0 <- p0 + theme_bw ()
  print (p0)

  dev.copy2pdf (file = paste0("reconpca", k, "_canad.pdf"), width = 12, height = 7)
}

# pouze pro jednu
df <- data.frame(dfs[1:365, ], reconstruction =  fdm)
p0 <- ggplot (data = df, aes (x = daytime, y = temp, group = place))
p0 <- p0 + geom_line (color = "grey", size = 0.5)
p0 <- p0 + geom_line (aes(y = reconstruction, color = place))
p0 <- p0 + labs (x = "Days", y = "Reconstructed Curve")
p0 <- p0 + scale_y_continuous(limits = c(-16, 20))
p0 <- p0 + theme_bw ()
print (p0)

dev.copy2pdf (file = paste0("meanone_canad.pdf"), width = 12, height = 7)


x11()
for (k in 1:6){
  df <- data.frame(dfs[1:365, ], reconstruction =  fdm + c(PCs[, 1:k] %*% t(scores[, 1:k]))[1:365])
  p0 <- ggplot (data = df, aes (x = daytime, y = temp, group = place))
  p0 <- p0 + geom_line (color = "grey", size = 0.5)
  p0 <- p0 + geom_line (aes(y = reconstruction, color = place))
  p0 <- p0 + labs (x = "Days", y = "Reconstructed Curve")
  p0 <- p0 + scale_y_continuous(limits = c(-16, 20))
  p0 <- p0 + theme_bw ()
  print (p0)
  
  dev.copy2pdf (file = paste0("recononepca", k, "_canad.pdf"), width = 12, height = 7)
}


# ##### 3. PCA of Multivariate Functions
# 
# # To look at two dimensions, we'll re-smooth the temperature data and look at 
# # it along with its derivative. To do that, let's consider a fourier basis
# # and harmonic acceleration. 
# 
# # First an fdPar object; we'll use 65 Fourier basis functions; more than enough
# # but this will cut down the computational time. We'll also over-smooth so our
# # derivatives don't look so wild. 
# 
# fbasis =  create.fourier.basis(dayrng,65)
# 
# harm.fdPar = fdPar(fbasis,harmLfd,1e6)
# 
# # Do the smooth
# tempSmooth2 = smooth.basis(daytime,temp,harm.fdPar)
# 
# # and take a derivative
# 
# temp.deriv = deriv.fd(tempSmooth2$fd)
# 
# # Now we need to duck under the hood to create a joine fd object for both
# # temperature and precipitation at once. 
# 
# # This basically means I need an fd object that stacks the coefficients for 
# # temperature and the coefficients for D temperature along a third dimension
# # of a coefficient array. 
# 
# Dtempcoefs = array(0,c(65,35,2))
# 
# Dtempcoefs[,,1] = tempSmooth2$fd$coefs
# Dtempcoefs[,,2] = temp.deriv$coefs
# 
# # Let's also deal with the dimension names 
# 
# Dtemp.fdnames = tempfd$fdnames
# Dtemp.fdnames[[3]] = c('temperature','D temperature')
# 
# # Put it all together
# 
# Dtempfd = fd(Dtempcoefs,fbasis,Dtemp.fdnames)
# 
# # Now we can plot
# 
# par(mfrow=c(2,1))
# plot(Dtempfd[,1])
# plot(Dtempfd[,2])
# 
# # We can also look at covariance
# 
# Dtemp.varbifd = var.fd(Dtempfd)
# 
# # If we look at 
# 
# Dtemp.var = eval.bifd(day5,day5,Dtemp.varbifd)
# 
# # We have a 74 x 74 x 1 x 3 array.
# 
# par(mfrow=c(1,1))
# 
# # First temperature
# contour(day5,day5,Dtemp.var[,,1,1])
# 
# # Then d temperature
# contour(day5,day5,Dtemp.var[,,1,3])
# 
# # Then their cross-product
# contour(day5,day5,Dtemp.var[,,1,2])
# 
# ### EXERCISE: experiment with cor.fd on this multidimensional fd object.
# 
# ## Now let's look at the PCA
# 
# Dtemp.pca = pca.fd(Dtempfd,nharm=4)
# 
# # The PCs are now two dimensional
# 
# par(mfrow=c(2,1))
# plot(Dtemp.pca$harmonics[,1])
# plot(Dtemp.pca$harmonics[,2])
# 
# # We can plot these in the usual manner
# 
# plot(Dtemp.pca,harm=1:3)
# 
# # But we can also plot the whole cycle
# 
# par(mfrow=c(1,1))
# plot(Dtemp.pca,harm=1:3,cycle=TRUE,xlab='temperature',ylab='D temperature')
# 
# # PC1 = over-all temperature and little variation in derivatives. 
# # PC2 = high summer-winter variation and also large variation in derivatives
# 


################################################################################
################################################################################
#                               Regression                                     #
################################################################################
################################################################################

# This lab will focus on functional linear models using the fRegress function.
# We will again make use of the Canadian Weather Data. We'll briefly re-do 
# the smooth here using Fourier bases:

data(CanadianWeather)

temp = CanadianWeather$dailyAv[,,1]
precip = CanadianWeather$dailyAv[,,2]

daytime = (1:365)-0.5
day5 = seq(0,365,5)

dayrng = c(0,365)

# Basis and parameter objects

fbasis =  create.fourier.basis(dayrng,65)
harmLfd = vec2Lfd(c(0,(2*pi/365)^2,0),rangeval=dayrng)
temp.fdPar = fdPar(fbasis,harmLfd,1e-2)

# And smooth

tempSmooth = smooth.basis(daytime,temp,temp.fdPar)

# And extract the functional data objects

tempfd = tempSmooth$fd


################################################################################
###                      FUNCTIONAL RESPONSE MODELS                         ####
################################################################################

# We could predict total precipitation from temperature. How about the 
# annual precipitation profile?

# We can also look at constant predictors -- see the weather demo for an ANOVA
# between different weather regions. 

# First we'll create a smooth of the log precipitation

prec.fdPar = fdPar(fbasis,harmLfd,1e4)
precSmooth = smooth.basis(daytime,log10(precip+0.5),prec.fdPar)
precfd = precSmooth$fd

# We can retain xlist from the scalar response model. 

xlist = list(len=2)

# First co-variate is just the intercept: a vector of ones

xlist[[1]] = rep(1,35)

# Second covariate is temperature

xlist[[2]] = tempfd

#### 1. fdPar objects and estimation

bwtlist2 = list(len=2)

# The intercept is now a functional parameter as well as beta 1.   Since this
# is an identifiable model without smoothing, we'll set the smoothing parameter 
# very low. 

beta.fdPar2 = fdPar(fbasis,harmLfd,1e-5)

bwtlist2[[1]] = beta.fdPar2
bwtlist2[[2]] = beta.fdPar2 

# We can also call fRegress with this

prec.conc = fRegress(precfd,xlist,bwtlist2)

# Let's have a look at what we've got

par(mfrow=c(2,1))
plot(prec.conc$betaestlist[[1]])
plot(prec.conc$betaestlist[[2]])

# We can also look at a comparison between predicted and observed

yhatfd  = prec.conc$yhatfdobj$fd  # Fitted smooths. 

plot(yhatfd)
plot(precfd)

plot(yhatfd[19])
lines(precfd[19], col = 2)


# And compare observed with residuals

plot(precfd-yhatfd)
plot(precfd)

# Doesn't look like we really got very much. 

df <- data.frame(daytime = daytime, temp = c(temp), precip = c(precip),
                 place = rep(CanadianWeather$place, each = 365), region = rep(CanadianWeather$region, each = 365))

df$precipfda <- c(eval.fd(precfd, evalarg = daytime))

# vykresleni srazek
x11()
p0 <- ggplot (data = df, aes (x = daytime, y = log10(precip), colour = place))
#p0 <- p0 + geom_point ()
#p0 <- p0 + guides (colour = FALSE) 
p0 <- p0 + geom_line ()
p0 <- p0 + labs (x = "Days", y = "Log10 of Precipitation") 
p0 <- p0 + theme_bw ()
#p0 <- p0 + scale_x_continuous (breaks = 1:365)
print (p0)

dev.copy2pdf (file = "log10precipitation.pdf", width = 12, height = 7)

# vykresleni vyhlazenych srazek
x11()
p0 <- ggplot (data = df, aes (x = daytime, y = precipfda, colour = place))
#p0 <- p0 + geom_point ()
#p0 <- p0 + guides (colour = FALSE) 
p0 <- p0 + geom_line ()
p0 <- p0 + labs (x = "Days", y = "Log10 of Precipitation") 
p0 <- p0 + theme_bw ()
#p0 <- p0 + scale_x_continuous (breaks = 1:365)
print (p0)

dev.copy2pdf (file = "log10precipitationfda.pdf", width = 12, height = 7)

# Vykresleni odhadu
df$preciphat <- c(eval.fd(yhatfd, evalarg = daytime))

x11()
p0 <- ggplot (data = subset(df, is.element(place, c("Churchill", "Regina"))), aes (x = daytime, y = precipfda, group = place))
p0 <- p0 + geom_line (color = "grey", size = 0.5)
p0 <- p0 + geom_line (aes(y = preciphat, color = place))
p0 <- p0 + labs (x = "Days", y = "Log10 of Precipitation")
p0 <- p0 + facet_wrap( ~ place)
p0 <- p0 + guides (colour = FALSE) 
p0 <- p0 + theme_bw ()
print (p0)

dev.copy2pdf (file = paste0("estim_precip.pdf"), width = 12, height = 7)


#### 2. Confidence Intervals

## In order to obtain confidence intervals, we can include the smoothing of 
## precipitation as a source of error. In order to do this, we need two things

y2cmap = precSmooth$y2cMap

# This is the matrix that goes from the observations to the coefficients. 

# We now need a covariance matrix for the errors of the original observed 
# precipitation from the functional linear model

Errmat = precip - eval.fd(daytime,yhatfd) 

SigmaE2 = cov(t(Errmat))

# We can now run fRegress.stderr

conc.std = fRegress.stderr(prec.conc,y2cmap,SigmaE2)

# And plot the results

plotbeta(prec.conc$betaestlist,conc.std$betastderrlist)

# There really doesn't appear to be much going on. 

beta0 <- eval.fd(prec.conc$betaestlist[[1]]$fd, evalarg = daytime)
beta1 <- eval.fd(prec.conc$betaestlist[[2]]$fd, evalarg = daytime)

betastderr0 <- eval.fd(conc.std$betastderrlist[[1]], evalarg = daytime)
betastderr1 <- eval.fd(conc.std$betastderrlist[[2]], evalarg = daytime)

betadf <- data.frame(x = c(daytime, daytime), beta = c(beta0, beta1), 
                     betastderr = c(betastderr0, betastderr1), 
                     type = factor(rep(c("Intercept", "Slope"), each = length(daytime)), ordered = TRUE))

x11()
p0 <- ggplot (data = betadf, aes (x = x, y = beta, group = type))
p0 <- p0 + geom_ribbon (aes (ymin = beta - 2*betastderr, ymax = beta + 2*betastderr), fill = "grey70")
p0 <- p0 + geom_line (size = 1.5)
p0 <- p0 + labs (x = "Days", y = "Beta(t)")
p0 <- p0 + facet_wrap( ~ type, scales = "free_y")
p0 <- p0 + guides (colour = FALSE) 
p0 <- p0 + theme_bw ()
print (p0)

dev.copy2pdf (file = paste0("bety.pdf"), width = 12, height = 7)


### EXERCISE: try predicting precipitation instead of log precipitation -- does
### this make a difference?

### EXERCISE: what diagnostics could be done to check goodness of fit? Try
### plotting residuals. Try plotting residuals against predicted values
### (this should give you a series of lines, you'll need to evaluate and use
### 'matplot').    How could you check for dependence of precipitation on
### non-concurrent times?
