### Ramsey & Silverman (2002) Applied Functional Data Analysis (Springer)
###
### ch. 3.  Nondurable Goods Index
###
library(fda)
library(ggplot2)
library(zoo)

x11()
plot(nondurables)

df <- data.frame(index = c(nondurables)[-973], month = month.abb, year = factor(rep(1919:1999, each = 12)))
df <- rbind(df, data.frame(index = c(nondurables)[973], month = "Jan", year = "2000"))
df$t <- seq(from = 1919, to = 2000, length.out = 973)

p <- ggplot(df, aes(x = t, y = index))
p <- p + geom_line()
p <- p + labs (x = "Time", y = "Nondurable Goods Index") 
p <- p + theme_bw ()
print(p)

dev.copy2pdf (file = "ngi.pdf", width = 12, height = 7)


# p. 43, Figure 3.2.  US nondurable goods production, log scale

# plot(log10(nondurables))


nondur1929 <- window(nondurables, 1929, 1930)
max1929 <- max(nondur1929)
max1929t <- which(nondur1929==max1929)

(Max1929 <- index(nondur1929)[max1929t])
# function 'index' is in 'zoo'

restrMoney <- window(nondurables, 1928, 1940)
maxRestr <- max(restrMoney)
maxRestr.t <- which(restrMoney==maxRestr)
(MaxRestr <- mean(index(restrMoney)[maxRestr.t]))
endVietnam <- (1975+4/12)
endVN <- min(window(nondurables, 1975, 1976))
(nondurFit <- lm(log10(nondurables)~index(nondurables)) )

p <- ggplot(df, aes(x = t, y = log10(index)))
p <- p + geom_line()
p <- p + labs (x = "Time", y = "Log10 of Index") 
p <- p + theme_bw ()
p <- p + geom_text(aes(x = Max1929, y = log10(max1929)+.01), hjust = 0, 
                   label = "Stock market crash", angle = 90, size = 5, col = 2)
p <- p + geom_text(aes(x = MaxRestr, y = log10(maxRestr)+.01), hjust = 0,
              label = "Restriction of money supply", angle = 90, size = 5, col = 2)
p <- p + geom_text(aes(x = endVietnam, y = log10(endVN)-0.01), hjust = 1, 
                   label = "End of Vietnam War", size = 5, angle = 90, col =3)
p <- p + geom_abline(slope = nondurFit$coefficients[2], intercept = nondurFit$coefficients[1], 
                     linetype = "dashed")

print(p)

dev.copy2pdf (file = "logngi.pdf", width = 12, height = 7)


# p. 44, Figure 3.3.  log nondurable goods index 1964 to 1967

#  Fit smooth per sec. 3.6.
goodsbasis <- create.bspline.basis(rangeval=c(1919,2000),
                                   nbasis=979, norder=8)

LfdobjNonDur <- int2Lfd(4);

#goodsfdPar = fdPar(goodsbasis, LfdobjNonDur, lambda=1e-11)
#lognondursmth = smooth.basis(durtime, coredata(lognondur), goodsfdPar);
logNondurSm <- smooth.basisPar(argvals=index(nondurables),
                               y=log10(coredata(nondurables)), fdobj=goodsbasis,
                               Lfdobj=LfdobjNonDur, lambda=1e-11)

#str(lognondursmth)

nondur1964.1967 <- window(nondurables, 1964, 1967)

# plot(log10(nondur1964.1967), type="p", axes=FALSE, xlab="Year",
#      ylab=expression(paste(log[10], " nondurable goods index")) )
# axis(2)
# axis(1, 1964:1967)
# axis(1, seq(1964, 1967, by=0.5), labels=FALSE)

#durtimefine = linspace(1964,1967,101);
durtimefine <- seq(1964, 1967, length=181)

#fit = eval.fd(durtimefine, lognondursmth);
logNondurSm1964.67 = eval.fd(durtimefine, logNondurSm$fd);
# lines(durtimefine, logNondurSm1964.67)
# abline(v=1965:1966, lty=2)

df1964 <- subset(df, is.element(year, 1964:1966) | (year == 1967 & month == "Jan"))

p <- ggplot()
p <- p + geom_point(data = df1964, aes(x = t, y = log10(index)))
p <- p + geom_line(aes(x = durtimefine, y = logNondurSm1964.67))
p <- p + labs (x = "Time", y = "Log10 of Index") 
p <- p + theme_bw ()
p <- p + geom_vline(xintercept = 1965:1966, linetype = "dashed")

print(p)

dev.copy2pdf (file = "logngi1964.pdf", width = 12, height = 7)

##
## sec. 3.3.  Phase-plane plots
##

#p. 45-46.  Figure 3.4.  Phase-plane plot of sin(2*pi*t)

sin. <- expression(sin(2*pi*x))
D.sin <- D(sin., "x")
D2.sin <- D(D.sin, "x")

# op <- par(pty="s")
# # square plot region so we get a circle not an ellipse
# with(data.frame(x=seq(0, 1, length=46)),
#      plot(eval(D.sin), eval(D2.sin), type="l",
#           xlim=c(-10, 10), ylim=c(-50, 50),
#           xlab="Velocity", ylab="Acceleration") )
# pi.2 <- (2*pi)
# #lines(x=c(-pi.2,pi.2), y=c(0,0), lty=3)
# abline(h=0, lty="longdash")
# pi.2.2 <- pi.2^2
# lines(x=c(0,0), y=c(-pi.2.2, pi.2.2), lty="longdash")
# 
# text(c(0,0), c(-47, 47), rep("no kinetic, max potential", 2))
# text(c(-8.5,8.5), c(0,0), rep("max kinetic\nno potential", 2))
# par(op)

x = seq(0, 1, length = 100)
dfsin <- data.frame(x = x, d1 = eval(D.sin), d2 = eval(D2.sin))

p <- ggplot()
p <- p + geom_path(data = dfsin, aes(x = d1, y = d2))
p <- p + labs (x = "Velocity", y = "Acceleration") 
p <- p + geom_text(aes(x = c(0, 0), y = c(-47, 47)), hjust = 0.42, 
                   label = "no kinetic, max potential", size = 5)
p <- p + geom_text(aes(x = c(-8.5, 8.5), y = c(0, 0)), hjust = 0.5,
                   label = "max kinetic\nno potential", size = 5)
p <- p + geom_vline(xintercept = 0, linetype = "longdash")
p <- p + geom_hline(yintercept = 0, linetype = "longdash")

p <- p + coord_fixed(ratio = 1/(2*pi), xlim = c(-10, 10), ylim = c(-50, 50))
p <- p + theme_bw ()
print(p)

dev.copy2pdf (file = "ppsin.pdf", width = 7, height = 7)


# sinusovka

x = seq(0, 1, length = 100)
dfsin <- data.frame(x = x, d1 = eval(sin.))

p <- ggplot()
p <- p + geom_path(data = dfsin, aes(x = x, y = d1))
p <- p + labs (x = "t", y = expression(sin(2~pi~t))) 
p <- p + geom_text(aes(x = c(0, 0.5) + 0.01, y = c(0, 0)), hjust = 0, 
                   label = "max kinetic\nno potential", size = 5)
p <- p + geom_text(aes(x = c(0.1, 0.6), y = c(1.07, -1.03)), hjust = 0,
                   label = "no kinetic, max potential", size = 5)
# p <- p + geom_vline(xintercept = 0, linetype = "longdash")
p <- p + geom_hline(yintercept = 0, linetype = "longdash")

#p <- p + coord_fixed(ratio = 1/(2*pi), xlim = c(-10, 10), ylim = c(-50, 50))
p <- p + theme_bw ()
print(p)

dev.copy2pdf (file = "sin.pdf", width = 7, height = 7)


##
## sec. 3.4.  Nondurable goods cycles
##
# Applied Functional Data Analysis, p. 56, sec. 3.6
# "Our final choice for lambda was 10^(-9.5)
# lognondur <- log10(nondurables) #???
# 
# 
# lam9.5 <- 10^(-9.5)
# goodsfdPar9.5 = fdPar(goodsbasis, LfdobjNonDur, lam9.5);
# #lognondursmth = smooth_basis(durtime, lognondur, goodsfdPar);
# lognondursm9.5 = smooth.basis(durtime, coredata(lognondur), goodsfdPar9.5);

# p. 47, Figure 3.5.  Nondurable phase-plane plot, 1964
##*** Need to add xlim and ylim to the following
## to match the plots in the book.

tt <- seq(1964, 1965, length.out = 361)

jkl <- phaseplanePlot(tt, logNondurSm$fd)
nn <- nrow(jkl)
dfjkl <- data.frame(d1 = jkl[- nn, 1], d2 = jkl[- nn, 2], year = "1964", month = rep(month.abb, each = 30))
dfjkl <- rbind(dfjkl, data.frame(d1 = jkl[nn, 1], d2 = jkl[nn, 2], year = "1965", month = "Jan"))

p <- ggplot()
p <- p + geom_path(data = dfjkl[1:181, ], aes(x = d1, y = d2), linetype = "dashed")
p <- p + geom_path(data = dfjkl[181:nn, ], aes(x = d1, y = d2))
p <- p + geom_vline(xintercept = 0, linetype = "longdash")
p <- p + geom_hline(yintercept = 0, linetype = "longdash")
p <- p + labs (x = "Velocity", y = "Acceleration") 
p <- p + geom_label(data = dfjkl[seq(1, nn, by = 30), ], aes(x = d1, y = d2, label = month))
p <- p + geom_label(data = dfjkl[1, ], aes(x = d1, y = d2, label = paste(month,year)), 
                    fill = "green4", fontface = "bold", col = "white")
p <- p + geom_label(data = dfjkl[nn, ], aes(x = d1 + 0.045, y = d2, label = year))

#p <- p + coord_fixed(ratio = 1/(2*pi), xlim = c(-10, 10), ylim = c(-50, 50))
p <- p + theme_bw ()
print(p)

dev.copy2pdf (file = "pp1964.pdf", width = 7, height = 7)


# obecne
roky <- c("1929", "1930", "1931", "1932")
chcuroky <- c("1929", "1930", "1931", "1932")

roky <- c("1937", "1938", "1939", "1940", "1941", "1942", "1943", "1944")
chcuroky <- c("1937", "1938", "1943")

roky <- c("1974", "1975", "1976","1977")
chcuroky <- c("1974", "1975", "1976")

roky <- c("1996", "1997", "1998","1999")
chcuroky <- c("1996", "1997", "1998")

tt <- seq(roky[1], roky[length(roky)], length.out = (length(roky) - 1)*360 + 1)

# rr <- data.frame(rok = as.numeric(roky))
# dd <- ddply(rr, .(rok), summarise, cc = seq(rok, rok + 1, length.out = 361)[-361])
# tt <- dd$cc
# tt[length(tt) + 1] <- as.numeric(roky)[length(roky)] + 1

jkl <- phaseplanePlot(tt, logNondurSm$fd)
nn <- nrow(jkl)
dfjkl <- data.frame(d1 = jkl[- nn, 1], d2 = jkl[- nn, 2], year = rep(roky[-length(roky)], each = 360), 
                    month = rep(month.abb, each = 30))
dfjkl <- rbind(dfjkl, data.frame(d1 = jkl[nn, 1], d2 = jkl[nn, 2], 
                                 year = roky[length(roky)], month = "Jan"))

dfjkl$gryear <- dfjkl$year
dfjkl$gryear[nrow(dfjkl)] <- dfjkl$gryear[nrow(dfjkl) - 1]

# dfjkl <- subset(dfjkl, is.element(year, chcuroky))
# 
# dfjkl$year <- droplevels(dfjkl$year)
# dfjkl$gryear <- droplevels(dfjkl$gryear)

nn <- nrow(dfjkl)

p <- ggplot(data = dfjkl, aes(group = year))
p <- p + geom_path(data = dfjkl[sort(rep(1:180, length(chcuroky)) + seq(0, length(chcuroky) + 1, by = 2)*180), ], aes(x = d1, y = d2), linetype = "dashed")
p <- p + geom_path(data = dfjkl[sort(rep(181:360, length(chcuroky)) + seq(0, length(chcuroky) + 1, by = 2)*180), ], aes(x = d1, y = d2))
p <- p + geom_vline(xintercept = 0, linetype = "longdash")
p <- p + geom_hline(yintercept = 0, linetype = "longdash")
p <- p + labs (x = "Velocity", y = "Acceleration") 
p <- p + geom_label(data = dfjkl[seq(1, nn-1, by = 30), ], aes(x = d1, y = d2, label = month))
p <- p + geom_label(data = dfjkl[seq(1, nn-1, by = 360), ], aes(x = d1, y = d2, label = month), 
                    fill = "green4", fontface = "bold", col = "white")
# p <- p + geom_label(data = dfjkl[1, ], aes(x = d1 + 0.045, y = d2, label = year))
# p <- p + geom_label(data = dfjkl[nn, ], aes(x = d1 + 0.045, y = d2, label = year))
p <- p + facet_wrap(~ gryear)
#p <- p + coord_fixed(ratio = 1/(2*pi), xlim = c(-10, 10), ylim = c(-50, 50))
p <- p + theme_bw ()
print(p)

dev.copy2pdf (file = "pp1929-1931.pdf", width = 12, height = 6)
dev.copy2pdf (file = "pp1937-1943.pdf", width = 12, height = 6)
dev.copy2pdf (file = "pp1974-1976.pdf", width = 12, height = 6)
dev.copy2pdf (file = "pp1996-1998.pdf", width = 12, height = 6)


# pp. 48-49, Figure 3.6.  Nondurable phase-plane plots, 1929-1931
phaseplanePlot(1929, logNondurSm$fd)
phaseplanePlot(1930, logNondurSm$fd)
phaseplanePlot(1931, logNondurSm$fd)

# pp. 48, 50, Figure 3.7.  Nondurable phase-plane plots, 1937-1938, 1943
phaseplanePlot(1937, logNondurSm$fd)
phaseplanePlot(1938, logNondurSm$fd)
phaseplanePlot(1939, logNondurSm$fd)

# pp. 48, 51, Figure 3.8.  Nondurable phase-plane plots, 1974-76
phaseplanePlot(1974, logNondurSm$fd)
phaseplanePlot(1975, logNondurSm$fd)
phaseplanePlot(1976, logNondurSm$fd)

# pp. 52-53, Figure 3.9.  Nondurable phase-plane plots, 1996-1998
phaseplanePlot(1996, logNondurSm$fd)
phaseplanePlot(1997, logNondurSm$fd)
phaseplanePlot(1998, logNondurSm$fd)

# p. 53, Figure 3.10.  Nondurable phase-plane plot, 1997, larger scale
phaseplanePlot(1997, logNondurSm$fd)

